searchData={"items":[{"type":"task","title":"mix cloak.migrate.ecto","doc":"Migrates a schema table to a new encryption cipher.","ref":"Mix.Tasks.Cloak.Migrate.Ecto.html"},{"type":"task","title":"Rationale - mix cloak.migrate.ecto","doc":"Cloak vaults will automatically decrypt fields which were encrypted\nby a retired key, and reencrypt them with the new key when they change.\n\nHowever, this usually is not enough for key rotation. Usually, you want\nto proactively reencrypt all your fields with the new key, so that the\nold key can be decommissioned.\n\nThis task allows you to do just that.","ref":"Mix.Tasks.Cloak.Migrate.Ecto.html#module-rationale"},{"type":"task","title":"Strategy - mix cloak.migrate.ecto","doc":"This task will migrate a table following this strategy:\n\n- Page through the table using a cursor, selecting primary key values. By\n  default, the cursor is based on the schema's primary key. To use custom\n  fields in the cursor, see `Cloak.CustomCursor`.\n\n  Here's an example SQL query generated by `mix cloak.migrate.ecto`:\n\n    ```sql\n    SELECT primary_key\n    FROM table_name\n    WHERE primary_key > cursor\n    ORDER BY primary_key ASC\n    LIMIT 100\n    ```\n\n- For each primary key value, in a database transaction,\n  - Fetch the row with that key value, lock with \"FOR UPDATE\"\n  - Reencrypt all Cloak fields with the new cipher\n  - Write the row, unlocking it\n\nUpdate queries are issued in parallel to maximize speed. Each row is fetched\nand written back as quickly as possible to reduce the amount of time the\nrow is locked.","ref":"Mix.Tasks.Cloak.Migrate.Ecto.html#module-strategy"},{"type":"task","title":"Warnings - mix cloak.migrate.ecto","doc":"1. `mix cloak.migrate.ecto` works well with primary keys of the following\n    types. If your key is not of these types, see `Cloak.CustomCursor`.\n\n   - Integers\n   - PostgreSQL UUID\n   - MongoDB ObjectID\n\n2. Because `mix cloak.migrate.ecto` issues queries in parallel, it can consume\n   all your database connections. For this reason, you may wish to use a\n   separate `Repo` with a limited `:pool` just for Cloak migrations. This will\n   allow you to prevent any performance impact by throttling Cloak to use only\n   a limited number of database connections.","ref":"Mix.Tasks.Cloak.Migrate.Ecto.html#module-warnings"},{"type":"task","title":"Configuration - mix cloak.migrate.ecto","doc":"Ensure that you have configured your vault to use the new cipher by default!\n\n    # If using mix configuration...\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.GCM, tag: \"NEW\", key: <<...>>},\n        retired: {Cloak.Ciphers.AES.CTR, tag: \"OLD\", key: <<...>>>}\n      ]\n\n    # If configuring in the `init/1` callback:\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n\n      @impl Cloak.Vault\n      def init(config) do\n        config =\n          Keyword.put(config, :ciphers, [\n            default: {Cloak.Ciphers.AES.GCM, tag: \"NEW\", key: <<...>>},\n            retired: {Cloak.Ciphers.AES.CTR, tag: \"OLD\", key: <<...>>>}\n          ])\n\n        {:ok, config}\n      end\n    end\n\nIf you want to migrate multiple schemas at once, you may find it convenient\nto specify the schemas in your `config/config.exs`:\n\n    config :my_app,\n      cloak_repo: [MyApp.Repo],\n      cloak_schemas: [MyApp.Schema1, MyApp.Schema2]","ref":"Mix.Tasks.Cloak.Migrate.Ecto.html#module-configuration"},{"type":"task","title":"Usage - mix cloak.migrate.ecto","doc":"To run against only a specific repo and schema, use the `-r` and `-s` flags:\n\n    mix cloak.migrate.ecto -r MyApp.Repo -s MyApp.Schema\n\nIf you've configured multiple schemas at once, as shown above, you can simply\nrun:\n\n    mix cloak.migrate.ecto","ref":"Mix.Tasks.Cloak.Migrate.Ecto.html#module-usage"},{"type":"behaviour","title":"Cloak.Ecto.CustomCursor","doc":"Defines a list of fields to use as a cursor when migrating a schema's table\nto a new encryption key using `mix cloak.migrate.ecto`.","ref":"Cloak.Ecto.CustomCursor.html"},{"type":"behaviour","title":"When to Use This - Cloak.Ecto.CustomCursor","doc":"This behaviour is useful in rare situations where your primary key cannot\nbe used as a cursor to page through the table. You need this if primary key\ncannot meet the \"Column Requirements\" below.\n\nFor example, you don't need to use this module if your primary key is:\n\n- An integer\n- A PostgreSQL UUID\n- A MongoDB ObjectID\n- Your primary key already meets the \"Column Requirements\"","ref":"Cloak.Ecto.CustomCursor.html#module-when-to-use-this"},{"type":"behaviour","title":"Column Requirements - Cloak.Ecto.CustomCursor","doc":"Each column you specify for the cursor must meet the following\nrequirements:\n\n- Each column must contain sortable values\n- Each column must be comparable using the `>` operator\n- The combined value of the columns must be unique in the table","ref":"Cloak.Ecto.CustomCursor.html#module-column-requirements"},{"type":"behaviour","title":"Example - Cloak.Ecto.CustomCursor","doc":"defmodule MyApp.MySchema do\n      use Ecto.Schema\n\n      @behaviour Cloak.Ecto.CustomCursor\n\n      schema \"table\" do\n        # ...\n      end\n\n      @impl Cloak.CustomCursor\n      def __cloak_cursor_fields__ do\n        [:sequential_id, :inserted_at]\n      end\n    end","ref":"Cloak.Ecto.CustomCursor.html#module-example"},{"type":"callback","title":"Cloak.Ecto.CustomCursor.__cloak_cursor_fields__/0","doc":"Returns a list of cursor fields.","ref":"Cloak.Ecto.CustomCursor.html#c:__cloak_cursor_fields__/0"},{"type":"callback","title":"Example - Cloak.Ecto.CustomCursor.__cloak_cursor_fields__/0","doc":"@impl Cloak.Ecto.CustomCursor\n    def __cloak_cursor_fields__ do\n      [:sequential_id, :inserted_at]\n    end","ref":"Cloak.Ecto.CustomCursor.html#c:__cloak_cursor_fields__/0-example"},{"type":"module","title":"Cloak.Ecto.Binary","doc":"An `Ecto.Type` to encrypt a binary field.","ref":"Cloak.Ecto.Binary.html"},{"type":"module","title":"Migration - Cloak.Ecto.Binary","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.Binary.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.Binary","doc":"Define an `Encrypted.Binary` module in your project:\n\n    defmodule MyApp.Encrypted.Binary do\n      use Cloak.Ecto.Binary, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.Binary\n    end","ref":"Cloak.Ecto.Binary.html#module-usage"},{"type":"module","title":"Cloak.Ecto.Date","doc":"An `Ecto.Type` to encrypt `Date` fields.","ref":"Cloak.Ecto.Date.html"},{"type":"module","title":"Migration - Cloak.Ecto.Date","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.Date.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.Date","doc":"Define an `Encrypted.Date` module in your project:\n\n    defmodule MyApp.Encrypted.Date do\n      use Cloak.Ecto.Date, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.Date\n    end","ref":"Cloak.Ecto.Date.html#module-usage"},{"type":"module","title":"Cloak.Ecto.DateTime","doc":"An `Ecto.Type` to encrypt `DateTime` fields.","ref":"Cloak.Ecto.DateTime.html"},{"type":"module","title":"Migration - Cloak.Ecto.DateTime","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.DateTime.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.DateTime","doc":"Define an `Encrypted.DateTime` module in your project:\n\n    defmodule MyApp.Encrypted.DateTime do\n      use Cloak.Ecto.DateTime, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.DateTime\n    end","ref":"Cloak.Ecto.DateTime.html#module-usage"},{"type":"module","title":"Cloak.Ecto.Decimal","doc":"An `Ecto.Type` to encrypt a decimal field, relying on the `Decimal`\nlibrary, which is a dependency of Ecto.\n\nDecimals are more precise than floats.","ref":"Cloak.Ecto.Decimal.html"},{"type":"module","title":"Migration - Cloak.Ecto.Decimal","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.Decimal.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.Decimal","doc":"Define an `Encrypted.Decimal` module in your project:\n\n    defmodule MyApp.Encrypted.Decimal do\n      use Cloak.Ecto.Decimal, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.Decimal\n    end","ref":"Cloak.Ecto.Decimal.html#module-usage"},{"type":"module","title":"Cloak.Ecto.Float","doc":"An `Ecto.Type` to encrypt a float field. Consider using `Cloak.Ecto.Decimal`\ninstead if precision is important.","ref":"Cloak.Ecto.Float.html"},{"type":"module","title":"Migration - Cloak.Ecto.Float","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.Float.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.Float","doc":"Define an `Encrypted.Float` module in your project:\n\n    defmodule MyApp.Encrypted.Float do\n      use Cloak.Ecto.Float, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.Float\n    end","ref":"Cloak.Ecto.Float.html#module-usage"},{"type":"behaviour","title":"Cloak.Ecto.HMAC","doc":"A custom `Ecto.Type` for hashing fields using `:crypto.hmac/3`.","ref":"Cloak.Ecto.HMAC.html"},{"type":"behaviour","title":"Why - Cloak.Ecto.HMAC","doc":"If you store a hash of a field's value, you can then query on it as a proxy\nfor an encrypted field. This works because HMAC is deterministic and\nalways results in the same value, while secure encryption does not. Be\nwarned, however, that hashing will expose which fields have the same value,\nbecause they will contain the same hash.","ref":"Cloak.Ecto.HMAC.html#module-why"},{"type":"behaviour","title":"Security - Cloak.Ecto.HMAC","doc":"HMAC is **more secure** than `Cloak.Ecto.SHA256`, because it uses a\nsecret to obfuscate the hash. This makes it harder to guess the value of\nthe field.","ref":"Cloak.Ecto.HMAC.html#module-security"},{"type":"behaviour","title":"Configuration - Cloak.Ecto.HMAC","doc":"Create an `HMAC` field in your project:\n\n    defmodule MyApp.Hashed.HMAC do\n      use Cloak.Ecto.HMAC, otp_app: :my_app\n    end\n\nThen, configure it with a `:secret` and `:algorithm`, either using\nmix configuration:\n\n    config :my_app, MyApp.Hashed.HMAC,\n      algorithm: :sha512,\n      secret: \"secret\"\n\nOr using the `init/1` callback to fetch configuration at runtime:\n\n    defmodule MyApp.Hashed.HMAC do\n      use Cloak.Ecto.HMAC, otp_app: :my_app\n\n      @impl Cloak.Ecto.HMAC\n      def init(config) do\n        config = Keyword.merge(config, [\n          algorithm: :sha512,\n          secret: System.get_env(\"HMAC_SECRET\")\n        ])\n\n        {:ok, config}\n      end\n    end","ref":"Cloak.Ecto.HMAC.html#module-configuration"},{"type":"behaviour","title":"Usage - Cloak.Ecto.HMAC","doc":"Create the hash field with the type `:binary`. Add it to your schema\ndefinition like this:\n\n    schema \"table\" do\n      field :field_name, MyApp.Encrypted.Binary\n      field :field_name_hash, MyApp.Hashed.HMAC\n    end\n\nEnsure that the hash is updated whenever the target field changes with the\n`put_change/3` function:\n\n    def changeset(struct, attrs \\\\ %{}) do\n      struct\n      |> cast(attrs, [:field_name, :field_name_hash])\n      |> put_hashed_fields()\n    end\n\n    defp put_hashed_fields(changeset) do\n      changeset\n      |> put_change(:field_name_hash, get_field(changeset, :field_name))\n    end\n\nQuery the Repo using the `:field_name_hash` in any place you would typically\nquery by `:field_name`.\n\n    user = Repo.get_by(User, email_hash: \"user@email.com\")","ref":"Cloak.Ecto.HMAC.html#module-usage"},{"type":"callback","title":"Cloak.Ecto.HMAC.init/1","doc":"Configures the `HMAC` field using runtime information.","ref":"Cloak.Ecto.HMAC.html#c:init/1"},{"type":"callback","title":"Example - Cloak.Ecto.HMAC.init/1","doc":"@impl Cloak.Fields.HMAC\n    def init(config) do\n      config = Keyword.merge(config, [\n        algorithm: :sha512,\n        secret: System.get_env(\"HMAC_SECRET\")\n      ])\n\n      {:ok, config}\n    end","ref":"Cloak.Ecto.HMAC.html#c:init/1-example"},{"type":"type","title":"Cloak.Ecto.HMAC.algorithms/0","doc":"HMAC algorithms supported by Cloak.Field.HMAC","ref":"Cloak.Ecto.HMAC.html#t:algorithms/0"},{"type":"module","title":"Cloak.Ecto.Integer","doc":"An `Ecto.Type` to encrypt integer fields.","ref":"Cloak.Ecto.Integer.html"},{"type":"module","title":"Migration - Cloak.Ecto.Integer","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.Integer.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.Integer","doc":"Define an `Encrypted.Integer` module in your project:\n\n    defmodule MyApp.Encrypted.Integer do\n      use Cloak.Ecto.Integer, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.Integer\n    end","ref":"Cloak.Ecto.Integer.html#module-usage"},{"type":"module","title":"Cloak.Ecto.IntegerList","doc":"An `Ecto.Type` to encrypt a list of integers.","ref":"Cloak.Ecto.IntegerList.html"},{"type":"module","title":"Configuration - Cloak.Ecto.IntegerList","doc":"You can customize the json library used for for converting the lists.\n\n    config :my_app, MyApp.Vault,\n      json_library: Jason","ref":"Cloak.Ecto.IntegerList.html#module-configuration"},{"type":"module","title":"Migration - Cloak.Ecto.IntegerList","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.IntegerList.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.IntegerList","doc":"Define an `Encrypted.IntegerList` module in your project:\n\n    defmodule MyApp.Encrypted.IntegerList do\n      use Cloak.Ecto.IntegerList, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.IntegerList\n    end","ref":"Cloak.Ecto.IntegerList.html#module-usage"},{"type":"module","title":"Cloak.Ecto.Map","doc":"An `Ecto.Type` to encrypt maps.","ref":"Cloak.Ecto.Map.html"},{"type":"module","title":"Configuration - Cloak.Ecto.Map","doc":"You can customize the json library used for for converting the maps to\nand from JSON during encryption and decryption.\n\n    config :my_app, MyApp.Vault,\n      json_library: Jason","ref":"Cloak.Ecto.Map.html#module-configuration"},{"type":"module","title":"Migration - Cloak.Ecto.Map","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.Map.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.Map","doc":"Define an `Encrypted.Map` module in your project:\n\n    defmodule MyApp.Encrypted.Map do\n      use Cloak.Ecto.Map, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.Map\n    end\n\nOn encryption, the map will first be converted to JSON using the configured\n`:json_library`, and then encrypted. On decryption, the `:json_library`\nwill be used to convert it back to a map.\n\nThis means that on decryption, atom keys will become string keys.\n\n    # ON WRITE\n    %{hello: \"world\"}\n\n    # ON READ\n    # Keys converted to strings\n    %{\"hello\" => \"world\"}","ref":"Cloak.Ecto.Map.html#module-usage"},{"type":"module","title":"Cloak.Ecto.NaiveDateTime","doc":"An `Ecto.Type` to encrypt `NaiveDateTime` fields.","ref":"Cloak.Ecto.NaiveDateTime.html"},{"type":"module","title":"Migration - Cloak.Ecto.NaiveDateTime","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.NaiveDateTime.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.NaiveDateTime","doc":"Define an `Encrypted.NaiveDateTime` module in your project:\n\n    defmodule MyApp.Encrypted.NaiveDateTime do\n      use Cloak.Ecto.NaiveDateTime, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.NaiveDateTime\n    end","ref":"Cloak.Ecto.NaiveDateTime.html#module-usage"},{"type":"behaviour","title":"Cloak.Ecto.PBKDF2","doc":"A custom `Ecto.Type` for deriving a key for fields using\n[PBKDF2](https://en.wikipedia.org/wiki/PBKDF2).\n\nPBKDF2 is **more secure** than `Cloak.Ecto.HMAC` and\n`Cloak.Fields.SHA256` because it uses [key\nstretching](https://en.wikipedia.org/wiki/Key_stretching) to increase the\namount of time to compute hashes. This slows down brute-force attacks.","ref":"Cloak.Ecto.PBKDF2.html"},{"type":"behaviour","title":"Why - Cloak.Ecto.PBKDF2","doc":"If you store a hash of a field's value, you can then query on it as a\nproxy for an encrypted field. This works because PBKDF2 is deterministic\nand always results in the same value, while secure encryption does not.\nBe warned, however, that hashing will expose which fields have the same\nvalue, because they will contain the same hash.","ref":"Cloak.Ecto.PBKDF2.html#module-why"},{"type":"behaviour","title":"Dependency - Cloak.Ecto.PBKDF2","doc":"To use this field type, you must install the `:pbkdf2` library in your\n`mix.exs` file.\n\n    {:pbkdf2, \"~> 2.0\"}\n\nIf you are using Erlang >= 24, you will need to use a forked version,\nbecause `pbkdf2` version `2.0.0` uses `:crypto.hmac` functions that were\nremoved in Erlang 24.\n\n    {:pbkdf2, \"~> 2.0\", github: \"miniclip/erlang-pbkdf2\"}","ref":"Cloak.Ecto.PBKDF2.html#module-dependency"},{"type":"behaviour","title":"Configuration - Cloak.Ecto.PBKDF2","doc":"Create a `PBKDF2` field in your project:\n\n    defmodule MyApp.Hashed.PBKDF2 do\n      use Cloak.Ecto.PBKDF2, otp_app: :my_app\n    end\n\nThen, configure it with a `:secret`, an `:algorithm`, the maximum `:size`\nof the stored key (in bytes), and a number of `:iterations`, either using\nmix configuration:\n\n    config :my_app, MyApp.Hashed.PBKDF2,\n      algorithm: :sha256,\n      iterations: 600_000,\n      secret: \"secret\",\n      size: 64\n\nOr using the `init/1` callback to fetch configuration at runtime:\n\n    defmodule MyApp.Hashed.PBKDF2 do\n      use Cloak.Ecto.PBKDF2, otp_app: :my_app\n\n      @impl Cloak.Ecto.PBKDF2\n      def init(config) do\n        config = Keyword.merge(config, [\n          algorithm: :sha256,\n          iterations: 600_000,\n          secret: System.get_env(\"PBKDF2_SECRET\")\n        ])\n\n        {:ok, config}\n      end\n    end","ref":"Cloak.Ecto.PBKDF2.html#module-configuration"},{"type":"behaviour","title":"Usage - Cloak.Ecto.PBKDF2","doc":"Create the hash field with the type `:binary`. Add it to your schema\ndefinition like this:\n\n    schema \"table\" do\n      field :field_name, MyApp.Encrypted.Binary\n      field :field_name_hash, MyApp.Hashed.PBKDF2\n    end\n\nEnsure that the hash is updated whenever the target field changes with the\n`put_change/3` function:\n\n    def changeset(struct, attrs \\\\ %{}) do\n      struct\n      |> cast(attrs, [:field_name, :field_name_hash])\n      |> put_hashed_fields()\n    end\n\n    defp put_hashed_fields(changeset) do\n      changeset\n      |> put_change(:field_name_hash, get_field(changeset, :field_name))\n    end\n\nQuery the Repo using the `:field_name_hash` in any place you would typically\nquery by `:field_name`.\n\n    user = Repo.get_by(User, email_hash: \"user@email.com\")","ref":"Cloak.Ecto.PBKDF2.html#module-usage"},{"type":"callback","title":"Cloak.Ecto.PBKDF2.init/1","doc":"Configures the `PBKDF2` field using runtime information.","ref":"Cloak.Ecto.PBKDF2.html#c:init/1"},{"type":"callback","title":"Example - Cloak.Ecto.PBKDF2.init/1","doc":"@impl Cloak.Ecto.PBKDF2\n    def init(config) do\n      config = Keyword.merge(config, [\n        algorithm: :sha256,\n        secret: System.get_env(\"PBKDF2_SECRET\")\n      ])\n\n      {:ok, config}\n    end","ref":"Cloak.Ecto.PBKDF2.html#c:init/1-example"},{"type":"type","title":"Cloak.Ecto.PBKDF2.algorithms/0","doc":"Digest algorithms supported by Cloak.Field.PBKDF2","ref":"Cloak.Ecto.PBKDF2.html#t:algorithms/0"},{"type":"module","title":"Cloak.Ecto.SHA256","doc":"An `Ecto.Type` which hashes the field value using the SHA256 algorithm.","ref":"Cloak.Ecto.SHA256.html"},{"type":"module","title":"Why - Cloak.Ecto.SHA256","doc":"If you store a hash of a field's value, you can then query on it as a proxy\nfor the encrypted field. This works because SHA256 is deterministic and\nalways results in the same value, while secure encryption does not. Be\nwarned, however, that hashing will expose which fields have the same value,\nbecause they will contain the same hash.","ref":"Cloak.Ecto.SHA256.html#module-why"},{"type":"module","title":"Security - Cloak.Ecto.SHA256","doc":"For a more secure hashing method, see one of the following alternatives:\n\n- `Cloak.Ecto.HMAC`\n- `Cloak.Ecto.PBKDF2`","ref":"Cloak.Ecto.SHA256.html#module-security"},{"type":"module","title":"Usage - Cloak.Ecto.SHA256","doc":"Create the hash field with the type `:binary`. Add it to your schema\ndefinition like this:\n\n    schema \"table\" do\n      field :field_name, MyApp.Encrypted.Binary\n      field :field_name_hash, Cloak.Ecto.SHA256\n    end\n\nEnsure that the hash is updated whenever the target field changes with the\n`put_change/3` function:\n\n    def changeset(struct, attrs \\\\ %{}) do\n      struct\n      |> cast(attrs, [:field_name, :field_name_hash])\n      |> put_change(:field_name_hash, get_field(changeset, :field_name))\n    end\n\nQuery the Repo using the `:field_name_hash` in any place you would typically\nquery by `:field_name`.\n\n    user = Repo.get_by(User, email_hash: \"user@email.com\")","ref":"Cloak.Ecto.SHA256.html#module-usage"},{"type":"module","title":"Cloak.Ecto.StringList","doc":"An `Ecto.Type` to encrypt a list of strings.","ref":"Cloak.Ecto.StringList.html"},{"type":"module","title":"Configuration - Cloak.Ecto.StringList","doc":"You can customize the json library used for for converting lists.\n\n    config :my_app, MyApp.Vault,\n      json_library: Jason","ref":"Cloak.Ecto.StringList.html#module-configuration"},{"type":"module","title":"Migration - Cloak.Ecto.StringList","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.StringList.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.StringList","doc":"Define an `Encrypted.StringList` module in your project:\n\n    defmodule MyApp.Encrypted.StringList do\n      use Cloak.Ecto.StringList, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.StringList\n    end\n\nOn encryption, the list will first be converted to JSON using the configured\n`:json_library`, and then encrypted. On decryption, the `:json_library` will\nbe used to convert it back to a list of strings.","ref":"Cloak.Ecto.StringList.html#module-usage"},{"type":"module","title":"Cloak.Ecto.Time","doc":"An `Ecto.Type` to encrypt `Time` fields.","ref":"Cloak.Ecto.Time.html"},{"type":"module","title":"Migration - Cloak.Ecto.Time","doc":"The database field must be of type `:binary`.\n\n    add :encrypted_field, :binary","ref":"Cloak.Ecto.Time.html#module-migration"},{"type":"module","title":"Usage - Cloak.Ecto.Time","doc":"Define an `Encrypted.Time` module in your project:\n\n    defmodule MyApp.Encrypted.Time do\n      use Cloak.Ecto.Time, vault: MyApp.Vault\n    end\n\nThen, define the type of your desired fields:\n\n    schema \"table_name\" do\n      field :encrypted_field, MyApp.Encrypted.Time\n    end","ref":"Cloak.Ecto.Time.html#module-usage"},{"type":"extras","title":"Cloak.Ecto","doc":"# Cloak.Ecto\n[![Hex Version](http://img.shields.io/hexpm/v/cloak_ecto.svg)](https://hex.pm/packages/cloak_ecto)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/cloak_ecto/)\n[![Total Download](https://img.shields.io/hexpm/dt/cloak_ecto.svg)](https://hex.pm/packages/cloak_ecto)\n[![Last Updated](https://img.shields.io/github/last-commit/danielberkompas/cloak_ecto.svg)](https://github.com/danielberkompas/cloak_ecto/commits/master)\n[![Build Status](https://danielberkompas.semaphoreci.com/badges/cloak_ecto/branches/master.svg?style=shields)](https://danielberkompas.semaphoreci.com/projects/cloak_ecto)\n[![Coverage Status](https://coveralls.io/repos/github/danielberkompas/cloak_ecto/badge.svg)](https://coveralls.io/github/danielberkompas/cloak_ecto)\n\nEasily encrypt fields in your [Ecto](https://github.com/elixir-ecto/ecto)\nschemas. Relies on [Cloak](https://github.com/danielberkompas/cloak) for\nencryption.\n\n- [Hex Documentation](https://hexdocs.pm/cloak_ecto)","ref":"readme.html"},{"type":"extras","title":"Usage - Cloak.Ecto","doc":"`Cloak.Ecto` helps you create `Ecto.Type` modules which automatically encrypt\nand decrypt your data. You simply define a type and set the type of your fields, and\n`Cloak.Ecto` handles the rest.\n\n```elixir\ndefmodule MyApp.Encrypted.Binary do\n  use Cloak.Ecto.Binary, vault: MyApp.Vault\nend\n```\n\n```elixir\ndefmodule MyApp.EctoSchema do\n  use Ecto.Schema\n\n  schema \"table_name\" do\n    field :encrypted_field, MyApp.Encrypted.Binary\n\n    # ...\n  end\nend\n```\n\nWhen Ecto writes the fields to the database, Cloak encrypts the values into a\nbinary blob, using a configured encryption algorithm chosen by you.\n\n```elixir\niex> Repo.insert!(%MyApp.EctoSchema{encrypted_field: \"plaintext\"})\n08:46:08.862 [debug] QUERY OK db=3.4ms\nINSERT INTO \"table_name\" (\"encrypted_field\") \nVALUES ($1) RETURNING \"id\", \"encrypted_field\" [\n  <<1,10, 65, 69, 83, 46, 67, 84, 82, 46, 86, 49, \n    69, 92, 173, 219, 203, 238, 26, 58, 236, 5, \n    104, 23, 12, 10, 182, 31, 221, 89, 22, 58, \n    34, 79, 109, 30, 70, 254, 56, 93, 102, 84>>\n]\n```\n\nLikewise, when Ecto reads the encrypted blob out of the database, Cloak will\nautomatically decrypt the value into the intended data type at runtime.\n\n```elixir\niex> Repo.get(MyApp.EctoSchema, 1)\n%MyApp.EctoSchema{encrypted_field: \"plaintext\"}\n```\n\nFor complete usage instructions, see the [Hex documentation](https://hexdocs.pm/cloak_ecto).","ref":"readme.html#usage"},{"type":"extras","title":"Troubleshooting - Cloak.Ecto","doc":"See our [troubleshooting guide](/guides/how_to/troubleshooting.md) for solutions to common issues.","ref":"readme.html#troubleshooting"},{"type":"extras","title":"Notable Features - Cloak.Ecto","doc":"- Transparent, easy to use encryption for database fields\n- Fully compatible with umbrella projects\n- Bring your own encryption algorithm, if you want\n- Mix task for key rotation: `mix cloak.migrate`","ref":"readme.html#notable-features"},{"type":"extras","title":"Security Notes - Cloak.Ecto","doc":"-  **Supported Algorithms**: Cloak's built-in encryption modules rely on Erlang's \n   `:crypto` module. Cloak supports the following algorithms out of the box:\n   \n    - AES.GCM\n    - AES.CTR\n\n- **Encrypted Data Not Searchable**: Cloak uses random IVs for each ciphertext. This \n  means that the same value will not encrypt to the same value twice. As a result,\n  encrypted columns are not queryable. However, Cloak does provide easy ways to\n  create hashed, searchable columns.\n\n- **Runtime Data is not Encrypted**: Cloak encrypts data _at rest_ in the database. \n  The data in your Ecto structs at runtime is not encrypted.\n\n- **No Support for User-specific Encryption Keys**: Cloak's `Ecto.Type` modules do not\n  support user-specific encryption keys, due to limitations on the `Ecto.Type` \n  behaviour. However, you can still use Cloak's ciphers to implement these in your \n  application logic.","ref":"readme.html#security-notes"},{"type":"extras","title":"Use Without Ecto - Cloak.Ecto","doc":"If you want to use `Cloak` without Ecto, see\n[`cloak`](https://hex.pm/packages/cloak) instead.","ref":"readme.html#use-without-ecto"},{"type":"extras","title":"Local Development - Cloak.Ecto","doc":"To develop this library locally, you will need to install the correct version\nof Elixir and Postgres. The easiest way to set everything up is with Docker\nand [docker-compose](https://docs.docker.com/compose/):\n\n```sh\n$ cd cloak_ecto\n# Runs the bin/test script in the context of Docker\n$ docker-compose run code bin/test\n# To access a terminal with mix, use this command:\n$ docker-compose run code bash\n# Run any command of your choosing:\nroot@234098234oij:/app# mix docs\n```","ref":"readme.html#local-development"},{"type":"extras","title":"Install Cloak","doc":"# Installing Cloak.Ecto\n\nThis guide will walk you through installing `Cloak.Ecto` in your project.\n\n#","ref":"install.html"},{"type":"extras","title":"Add the Dependency - Install Cloak","doc":"First, add `:cloak_ecto` to your dependencies in `mix.exs`:\n\n    {:cloak_ecto, \"~> 1.2.0\"}\n\nRun `mix deps.get` to fetch the dependency. Since `:cloak_ecto` relies\non `:cloak`, the `:cloak` package will also be installed.\n\n#","ref":"install.html#add-the-dependency"},{"type":"extras","title":"Generate a Key - Install Cloak","doc":"You'll need a secret key for encryption. This is easy to generate in the\nIEx console.\n\n    $ iex\n    iex> 32 |> :crypto.strong_rand_bytes() |> Base.encode64()\n    \"aJ7HcM24BcyiwsAvRsa3EG3jcvaFWooyQJ+91OO7bRU=\"\n\nThis will generate a relatively strong encryption 256-bit encryption\nkey encoded with Base64.\n\n#","ref":"install.html#generate-a-key"},{"type":"extras","title":"Create a Vault - Install Cloak","doc":"Next, create a `Cloak.Vault` for your project.\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n    end\n\nConfigure it as shown in the `Cloak.Vault` documentation, with at least one\nactive cipher. Note that the `:key` needs to be decoded from Base64 encoding into\nits raw binary form.\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: Base.decode64!(\"your-key-here\")}\n      ]\n\nIf you want to fetch keys from system vars, you should use the `init/1` callback\nto configure the vault instead:\n\n    # Assumes that you have a CLOAK_KEY environment variable containing a key in\n    # Base64 encoding.\n    #\n    # export CLOAK_KEY=\"A7x+qcFD9yeRfl3GohiOFZM5bNCdHNu27B0Ozv8X4dE=\"\n\n    defmodule MyApp.Vault do\n      use Cloak.Vault, otp_app: :my_app\n\n      @impl GenServer\n      def init(config) do\n        config =\n          Keyword.put(config, :ciphers, [\n            default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: decode_env!(\"CLOAK_KEY\")}\n          ])\n\n        {:ok, config}\n      end\n\n      defp decode_env!(var) do\n        var\n        |> System.get_env()\n        |> Base.decode64!()\n      end\n    end\n\nFinally, add your vault to your supervision tree.\n\n    children = [\n      MyApp.Vault\n    ]\n\n#","ref":"install.html#create-a-vault"},{"type":"extras","title":"Create Local Ecto Types - Install Cloak","doc":"For each type of data you want to encrypt, define a local Ecto type:\n\n    defmodule MyApp.Encrypted.Binary do\n      use Cloak.Ecto.Binary, vault: MyApp.Vault\n    end\n\nYou can find a complete list of available types in the \"MODULES\" documentation.\n\n#","ref":"install.html#create-local-ecto-types"},{"type":"extras","title":"Create Your Schema - Install Cloak","doc":"If you want to encrypt an existing schema, see the guide on [Encrypting\nExisting Data](encrypt_existing_data.html).\n\nIf you're starting from scratch with a new `Ecto.Schema`, generate your\nfields with the type `:binary`:\n\n    create table(:users) do\n      add :email, :binary\n      add :email_hash, :binary # will be used for searching\n      # ...\n\n      timestamps()\n    end\n\nYour schema module should look like this:\n\n    defmodule MyApp.Accounts.User do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :email, MyApp.Encrypted.Binary\n        field :email_hash, MyApp.Hashed.HMAC\n        # ... other fields\n\n        timestamps()\n      end\n\n      @doc false\n      def changeset(struct, attrs \\\\ %{}) do\n        struct\n        |> cast(attrs, [:email])\n        |> put_hashed_fields()\n      end\n\n      defp put_hashed_fields(changeset) do\n        changeset\n        |> put_change(:email_hash, get_field(changeset, :email))\n      end\n    end\n\nThis example also shows how you would make a given field queryable by\ncreating a mirrored `_hash` field. See `Cloak.Ecto.HMAC` for more details.","ref":"install.html#create-your-schema"},{"type":"extras","title":"Usage - Install Cloak","doc":"Your encrypted fields will be transparently encrypted and decrypted as\ndata are loaded from the database.\n\n    Repo.get(Accounts.User, 1)\n    # => %Accounts.User{email: \"test@example.com\", email_hash: <<115, 6, 45, 135, 41, ...>>}\n\nYou can query by the mirrored `_hash` fields:\n\n    Repo.get_by(Accounts.User, email_hash: \"test@example.com\")\n    # => %Accounts.User{email: \"test@example.com\", ...}\n\nAnd you're done! `Cloak.Ecto` is successfully installed.","ref":"install.html#usage"},{"type":"extras","title":"Generate Encryption Keys","doc":"# How to Generate Encryption Keys\n\nThe easiest way to generate a key is via IEx.\n\n    $ iex\n    iex> 32 |> :crypto.strong_rand_bytes() |> Base.encode64()\n    \"HXCdm5z61eNgUpnXObJRv94k3JnKSrnfwppyb60nz6w=\"\n\nThis will generate a strong 256-bit key encoded with Base64.","ref":"generate_keys.html"},{"type":"extras","title":"Encrypt Existing Data","doc":"# How to Encrypt Existing Data\n\nWhen you adopt Cloak in an existing project, you often need to encrypt fields\nthat were not previously encrypted. This guide will help you through the\nprocess, step by step.","ref":"encrypt_existing_data.html"},{"type":"extras","title":"Create a Vault - Encrypt Existing Data","doc":"Make sure you've set up (and configured!) a vault as described in the\n`Cloak.Vault` documentation and [the installation guide](install.html).","ref":"encrypt_existing_data.html#create-a-vault"},{"type":"extras","title":"Create Local Ecto Types - Encrypt Existing Data","doc":"Create local Ecto types for each kind of field you want to encrypt, as\ndescribed in the `Cloak.Vault` documentation and [the installation\nguide](install.html).","ref":"encrypt_existing_data.html#create-local-ecto-types"},{"type":"extras","title":"Add `encrypted_*` Fields - Encrypt Existing Data","doc":"You'll need to create a duplicate version of each field you want to encrypt,\nperhaps with the `encrypted_` prefix.\n\n    alter table(:users) do\n      add :encrypted_name, :binary\n      add :encrypted_metadata, :binary\n    end\n\nThen, add the new fields to your schema. Make sure to keep them up to\ndate with changes using a temporary `put_encrypted_fields` function in\nyour changeset.\n\n    defmodule MyApp.Accounts.User do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :name, :string\n        field :encrypted_name, MyApp.Encrypted.Binary\n        field :metadata, :map\n        field :encrypted_metadata, MyApp.Encrypted.Map\n      end\n\n      @doc false\n      def changeset(struct, attrs \\\\ %{}) do\n        struct\n        |> cast(attrs, [:name, :metadata])\n        |> put_encrypted_fields()\n      end\n\n      # Temporary function during the migration process, to ensure\n      # that all changes are copied over to the new fields\n      defp put_encrypted_fields(changeset) do\n        changeset\n        |> put_change(:encrypted_name, get_field(changeset, :name))\n        |> put_change(:encrypted_metadata, get_field(changeset, :metadata))\n      end\n    end","ref":"encrypt_existing_data.html#add-encrypted_-fields"},{"type":"extras","title":"Copy Data to New Fields - Encrypt Existing Data","doc":"In a custom mix task or the IEx console, run the following:\n\n    MyApp.Accounts.User\n    |> MyApp.Repo.all()\n    |> Enum.map(fn user ->\n      user\n      |> Ecto.Changeset.change(%{\n           encrypted_name: user.name,\n           encrypted_metadata: user.metadata\n         })\n      |> MyApp.Repo.update!()\n    end)","ref":"encrypt_existing_data.html#copy-data-to-new-fields"},{"type":"extras","title":"Remove Original Fields - Encrypt Existing Data","doc":"Once you're confident that all the data has migrated successfully to the new\nfields, you can write a migration to remove the old, unencrypted fields.\n\n    alter table(:users) do\n      remove :name\n      remove :metadata\n    end\n\n    rename table(:users), :encrypted_name, to: :name\n    rename table(:users), :encrypted_metadata, to: :metadata\n\nYou can also remove the temporary `put_encrypted_fields/1` function, leaving\nyour schema like this:\n\n    defmodule MyApp.Accounts.User do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      schema \"users\" do\n        field :name, MyApp.Encrypted.Binary\n        field :metadata, MyApp.Encrypted.Map\n      end\n\n      @doc false\n      def changeset(struct, attrs \\\\ %{}) do\n        struct\n        |> cast(attrs, [:name, :metadata])\n      end\n    end","ref":"encrypt_existing_data.html#remove-original-fields"},{"type":"extras","title":"Rotate Keys","doc":"# How to Rotate Keys\n\nMany organizations rotate their keys on a regular schedule for security\nreasons. Cloak can do this with **zero downtime**.","ref":"rotate_keys.html"},{"type":"extras","title":"Add Your New Key - Rotate Keys","doc":"You will need to add your new key to your vault configuration, **as the\nfirst key in the `:ciphers` list.** Demote and re-label existing keys.\n\nFor example, if your config currently looks like this:\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: <<...>>},\n      ]\n\nThen you should change the `:default` label to the new key, and demote the\nexisting key to the `:retired` label.\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V2\", key: <<...>>},\n        retired: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V1\", key: <<...>>}\n      ]\n\n> ##","ref":"rotate_keys.html#add-your-new-key"},{"type":"extras","title":"Labels don't matter, order does {: .tip} - Rotate Keys","doc":">\n> The `:ciphers` configuration is _order-dependent_. The first key in the list\n> will be used as the default, regardless of its label. This guide uses the \n> `:default` and `:retired` labels for clarity, but the order is what matters.","ref":"rotate_keys.html#labels-don-t-matter-order-does-tip"},{"type":"extras","title":"Migrate Data To New Key - Rotate Keys","doc":"For each schema that uses your vault, run `mix cloak.migrate`:\n\n    mix cloak.migrate.ecto -r MyApp.Repo -s MyApp.Schema\n\nAlternatively, you can migrate multiple schemas at once by configuring\nthe following values in `config/config.exs`:\n\n    config :my_app,\n      cloak_repo: MyApp.Repo,\n      cloak_schemas: [...]\n\nWith that in place, you can simply run:\n\n    mix cloak.migrate.ecto","ref":"rotate_keys.html#migrate-data-to-new-key"},{"type":"extras","title":"Remove Retired Key - Rotate Keys","doc":"Once you are confident that all data have been converted over to use\nthe new key, you may remove the `:retired` key from your configuration.\n\n    config :my_app, MyApp.Vault,\n      ciphers: [\n        default: {Cloak.Ciphers.AES.GCM, tag: \"AES.GCM.V2\", key: <<...>>},\n      ]\n\n# How It Works\n\nEvery ciphertext generated by Cloak is tagged with the `:tag` of the\nconfigured cipher that produced it. \n\nAs a result, Cloak will be able to find and use the correct cipher to decrypt\nany old ciphertext as long as you leave its cipher configured.\n\nWhen you add a new default key, Cloak will immediately begin encrypting\nnew data with the new key. Every time existing data are fetched from the\ndatabase, they will be decrypted with the old key, and if changed, will\nbe encrypted with the new key.\n\nEven if you don't run `mix cloak.migrate.ecto`, data will gradually be\nconverted over to the new key as they are used. The migration task simply\nspeeds up the process.","ref":"rotate_keys.html#remove-retired-key"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting\n\n#","ref":"troubleshooting-1.html"},{"type":"extras","title":"Error when validating default values - Troubleshooting","doc":"If you have a field in your schema with a `:default` value, and are running Ecto\n3.6 or later, you may encounter an error like this:\n\n```\n** (ArgumentError) errors were found at the given arguments:\n\n  * 1st argument: the table identifier does not refer to an existing ETS table\n\n    (stdlib 4.0) :ets.lookup(MyApp.Vault.Config, :config)\n    (cloak 1.1.2) lib/cloak/vault.ex:285: Cloak.Vault.read_config/1\n    lib/cloak/vault.ex:224: MyApp.Vault.encrypt/1\n    lib/cloak_ecto/type.ex:37: Cloak.Ecto.Encrypted.Binary.dump/1\n    (ecto 3.8.4) lib/ecto/schema.ex:2198: Ecto.Schema.validate_default!/3\n    (ecto 3.8.4) lib/ecto/schema.ex:1919: Ecto.Schema.__field__/4\n    lib/my_app/post.ex:9: (module)\n```\n\nThis error happens because `cloak` relies on an ETS table which is only available\nat runtime, and Ecto 3.6+ tries to validate default values at compile time.\n\nTo fix it, add the `:skip_default_validation` option:\n\n```elixir\nfield :name, MyApp.Encrypted.Binary, default: \"foo\", skip_default_validation: true\n```","ref":"troubleshooting-1.html#error-when-validating-default-values"},{"type":"extras","title":"Closure Wrapping","doc":"# Closure wrapping\n\nUse the `closure: true` option in local Ecto type definitions to wrap decrypted\nvalues in a zero-arity closure. This prevents the plaintext from accidentally\nleaking in some contexts, such as in stacktraces, `Inspect` documents and JSON\nserializations.\n\n    defmodule MyApp.Encrypted.WrappedBinary do\n      use Cloak.Ecto.Binary, vault: MyApp.Vault, closure: true\n    end\n\nIn those places where the plaintext value is needed, unwrap the value by\ninvoking the closure as an anonymous function. Consider using\n[Plug.Crypto.prune_args_from_stacktrace/1](https://hexdocs.pm/plug_crypto/Plug.Crypto.html#prune_args_from_stacktrace/1)\nin functions that unwrap the plaintext, to prevent leakage in exceptions that\nmay occur further down the call stack:\n\n    def basic_auth(req, client) do\n      # Unwrap string value from password closure\n      HTTPClient.basic_auth(req, client.username, client.password.())\n    rescue\n      e ->\n        reraise e, Plug.Crypto.prune_args_from_stacktrace(System.stacktrace())\n    end","ref":"closure_wrapping.html"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting\n\n#","ref":"troubleshooting-2.html"},{"type":"extras","title":"Error when validating default values - Troubleshooting","doc":"If you have a field in your schema with a `:default` value, and are running Ecto\n3.6 or later, you may encounter an error like this:\n\n```\n** (ArgumentError) errors were found at the given arguments:\n\n  * 1st argument: the table identifier does not refer to an existing ETS table\n\n    (stdlib 4.0) :ets.lookup(MyApp.Vault.Config, :config)\n    (cloak 1.1.2) lib/cloak/vault.ex:285: Cloak.Vault.read_config/1\n    lib/cloak/vault.ex:224: MyApp.Vault.encrypt/1\n    lib/cloak_ecto/type.ex:37: Cloak.Ecto.Encrypted.Binary.dump/1\n    (ecto 3.8.4) lib/ecto/schema.ex:2198: Ecto.Schema.validate_default!/3\n    (ecto 3.8.4) lib/ecto/schema.ex:1919: Ecto.Schema.__field__/4\n    lib/my_app/post.ex:9: (module)\n```\n\nThis error happens because `cloak` relies on an ETS table which is only available\nat runtime, and Ecto 3.6+ tries to validate default values at compile time.\n\nTo fix it, add the `:skip_default_validation` option:\n\n```elixir\nfield :name, MyApp.Encrypted.Binary, default: \"foo\", skip_default_validation: true\n```","ref":"troubleshooting-2.html#error-when-validating-default-values"},{"type":"extras","title":"0.9.x to 1.0.x","doc":"# How To Upgrade From 0.9.x to 1.0.x\n\nCloak 1.0.0 separated the `Ecto` components into a separate package:\n`cloak_ecto`. This required some breaking name changes.\n\n- Cloak's default JSON library is now `Jason`, not `Poison`\n- All `Ecto`-related modules now have the prefix `Cloak.Ecto`.\n- Cloak's `Ecto` types have a new naming scheme: `Cloak.Fields.Binary` became\n  `Cloak.Ecto.Binary`.\n- `mix cloak.migrate` has been renamed to `mix cloak.migrate.ecto`.\n\n#","ref":"0-9-x_to_1-0-x.html"},{"type":"extras","title":"Install `cloak_ecto` 1.0.0 - 0.9.x to 1.0.x","doc":"Replace your `cloak` dependency with `cloak_ecto`.\n\n    {:cloak_ecto, \"1.0.0\"}\n\n#","ref":"0-9-x_to_1-0-x.html#install-cloak_ecto-1-0-0"},{"type":"extras","title":"Configure JSON Library - 0.9.x to 1.0.x","doc":"If you have not specified the `:json_library` setting in your Vault, and want\nto continue using Poison, you will need to add that setting:\n\n    config :my_app, MyApp.Vault,\n      json_library: Poison,\n      # ...\n\nOtherwise, you will need to add `Jason` to your dependencies:\n\n    {:jason, \"~> 1.0\"}\n\n#","ref":"0-9-x_to_1-0-x.html#configure-json-library"},{"type":"extras","title":"Update Cloak Field References - 0.9.x to 1.0.x","doc":"Update all your Cloak field references from the old naming scheme to the new.\nFor example:\n\n    use Cloak.Fields.Binary, vault: MyApp.Vault\n\nShould now become:\n\n    use Cloak.Ecto.Binary, vault: MyApp.Vault\n\n#","ref":"0-9-x_to_1-0-x.html#update-cloak-field-references"},{"type":"extras","title":"Update Migration References - 0.9.x to 1.0.x","doc":"If you use `Cloak.Migrator` directly, rename your references to\n`Cloak.Ecto.Migrator` instead. If you use `mix cloak.migrate`, rename any\nreferences to `mix cloak.migrate.ecto` instead..","ref":"0-9-x_to_1-0-x.html#update-migration-references"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}